function [x_real_to] = diff_drive_process_step(x_real_from, real_control_input, real_process_noise)
% performs a 'real' step

    
    % x_real_from is expected to be a structure with value and uncert [x y phi]'
    % control_input is expected to be [delta d, delta phi]'
    
    x_old = x_real_from.value(1,1);
    y_old = x_real_from.value(2,1);
    phi_old = x_real_from.value(3,1);
    
    delta_D = real_control_input.value(1,1);
    delta_phi = real_control_input.value(2,1);
    
    x_real_to.value(1,1) = x_old + delta_D*cos(phi_old + (delta_phi/2)) ;
    x_real_to.value(2,1) = y_old + delta_D*sin(phi_old + (delta_phi/2)) ;
    x_real_to.value(3,1) = phi_old + delta_phi;
         
    
    % add real Gaussian process noise
    [m n] = size(x_real_from.uncert);
    for i=1:m, x_real_to.value(i) = x_real_to.value(i,1) + sqrt(real_process_noise.uncert(i,i))*randn; end;
    
    for i=1:m, for j=1:m, x_real_to.uncert(i,j) = x_real_from.uncert(i,j); end; end;
        
    %disp 'real process step made.';

   