
function [x, y, major, minor, bbox] = draw_ellipse_make (covar, level, n)

  if (nargin < 3)
    n = 181;        % number of points for x,y array
  end

  if (nargin > 1)

    [v, l] = eig (covar);    % eigen vector and values of covar matrix amat

    dl = diag(l);           % eigen values are on diagonal of l
    if (any (imag (dl)) || any (dl <= 0))
      error ('ellipse: covar must be positive definite');
    end

    % generate ellips data

    a = level * sqrt (l(1,1));  % level is nr of sigmas, level = 1 => 1 sigma
    b = level * sqrt (l(2,2));  % sqrt of the eigenvalue = length of axis

    t = linspace (0, 2*pi, n)';

    xt = a * cos (t);           % plot the ellips
    yt = b * sin (t);

    %   rotate the contours

    ra = atan2 (v(2,1), v(1,1));   

    cos_ra = cos (ra);  % calculate angles
    sin_ra = sin (ra);

    x = xt * cos_ra - yt * sin_ra; % rotate points
    y = xt * sin_ra + yt * cos_ra;

    %  endpoints of the major and minor axes

    major = (v * diag ([a, b]))';
    minor = (v * diag ([a, b]))';

    major (2,:) = -major (1,:);
    minor (1,:) = -minor (2,:);

   % ## Bounding box for the ellipse using magic formula.

    ainv = inv (covar);
    xbox = sqrt (level * ainv(1,1));
    ybox = sqrt (level * ainv(2,2));

    bbox = [xbox ybox; xbox -ybox; -xbox -ybox; -xbox ybox; xbox ybox];

  else
    usage ('ellipse (covar, level, n)');
    end

end
%#########################################################
