function [x_post, C_x_post, K] = iekf_meas_update_step(prob)

    % extract parameters and Jacobians etc from model
    [h n para J] = get_model_settings(prob, prob.model.meas);
   
    % assuming the first argument is the old state (!)
    C_x_prior = para(1).uncert;
    x_prior = para(1).value;
 
    
    % get last made real measurement 
    [z] = prob.env.meas.result.value;
    
    %if no real measurement, then 
    if isnan(z(1,1))
        x_post = x_prior;
        C_x_post = C_x_prior;
        return
    end
    
    
    % iteration part
    % 
    go_on = 1;
    
    while go_on > 0,
        
        % get latest state estimate (amongst others)
        % extract parameters and Jacobians etc from model
        [h n para J] = get_model_settings(prob, prob.model.meas);
  
        
        % make measurement prediction, based on this last state estimate
        [z_hat] = feval(h, para(:).value);
    
    
        % calculate measurement residual
        [z_res] = z - z_hat;
    
    
        % calculate uncertainty in residual
        % this is noise in prediction plus measurement
        for i=1:n,
            C_tmp(i).influence = J(i).m*para(i).uncert*(J(i).m');
        end;
        
        C_res = C_tmp(1).influence;
        if n>1, for i=2:n, C_res = C_res + C_tmp(i).influence; end; end;
    
    
        % ready for updates...
         
        % calculate Kalman gain   
        K = (C_x_prior*((J(1).m)'))*( C_res^(-1) );
    
    
        % update state
        x_post = x_prior + K*z_res;
        
        % see if difference is significant with last estimate
        x_diff = prob.model.state_est.value - x_post;
        
        signif = 0;
        
        [n m] = size(x_diff);
        
        for i=1:n,
            if abs(x_diff(i,1)) > 0.00005,
                signif = 1;
            end;
        end;
        
        % save x_post in prob structure, such that a new 
        % iteration with x_post as new state estimate can be done
        prob.model.state_est.value = x_post;
    
        % iterate until state does not change significantly
        % or until x rounds made
        if (go_on < 100 && signif == 1),
            go_on = go_on + 1;
        else
            go_on = 0;
        end;
        
        display_info(prob);
    %    disp 'press key..';
       % pause;
        
    end;
    
   
    % now also update the uncertainty
    C_x_post = C_x_prior - K*(J(1).m)*C_x_prior;
    
    % done.
    pr 'iekf measurement update performed.';

