function [figures] = charts_estimate(kf_data, steps, figs)

    if nargin < 1,
        disp 'syntax: charts_estimate(kf_data, nr_of_steps_to_show, [figures])';
        error('not enough parameters');
    elseif nargin == 1,
        steps = 0;        
    end;
    
	% display charts for estimates

%    figures = figs;
    
    % prior state trajectories
    figures.est_prior_state_trajectories = figure;
            
    [prior_val prior_uncert] = convert_step_data_2_arr(kf_data.x_prior);
    arr_source = prior_val;
    
    
    [to c] = size(arr_source);
    state_size = c-1;
    real_state_size = state_size; %for later use in uncertainties..
    
    
    if ~(steps==0),
        to = steps;
    end;
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_prior_state_trajectory(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1))
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory prior state element ',num2str(i)]);
	
    end
    
	
    % posterior state trajectories
    figures.est_post_state_trajectories = figure;
            
    [post_val post_uncert] = convert_step_data_2_arr(kf_data.x_post);
    arr_source = post_val;
    
    
    [r c] = size(arr_source);
    state_size = c-1;
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_post_state_trajectory(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1))
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory post state element ',num2str(i)]);
	
    end
    
    
    % measurement estimate trajectories
    figures.est_meas_trajectories = figure;
            
    [z_hat_val z_hat_uncert] = convert_step_data_2_arr(kf_data.z_hat);
    arr_source = z_hat_val;
  
    
    [r c] = size(arr_source);
    state_size = c-1;
    real_meas_size = state_size;
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_meas_trajectory(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1),'o')
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory meas element ',num2str(i)]);
	
    end
    
    
    % measurement estimate trajectories
    figures.est_meas_res_trajectories = figure;
            
    [z_res_val z_res_uncert] = convert_step_data_2_arr(kf_data.z_res);
    arr_source = z_res_val;
  
    [r c] = size(arr_source);
    state_size = real_meas_size;
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_meas_res_trajectory(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1),'o')
    	xlabel 'step';
    	ylabel 'value';
    	title (['trajectory meas residual element ',num2str(i)]);
	
    end
    
    
    % figure with meas residual uncertainties
    
    figures.meas_res_uncertainties = figure;
            
    arr_source = z_res_uncert; % from above...
    
    
%    [r c] = size(arr_source);
%    state_size = c-1;
    
    fig_width = real_meas_size;
    fig_height = real_meas_size;
    
    
    for i=1:real_meas_size*real_meas_size
    
    
	    % display data of element i
	    figures.meas_res_uncertanty(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1))
    	xlabel 'step';
    	ylabel '(co)variance';
    	title (['meas residual uncertainty in element ',num2str(i)]);
	
    end
   
    
    figures.meas_res_uncert_plus_residual = figure;
    fig_width = 1;
    fig_height = state_size;
 
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_meas_res_trajectory(i) = subplot(fig_height,fig_width,i);
        
	    plot(z_res_val(1:to,1),z_res_val(1:to,i+1),'o')
        hold on;
        plot(z_res_uncert(1:to,1),2*sqrt(z_res_uncert(1:to, i+1 + (i-1)*state_size)),'-');
    	plot(z_res_uncert(1:to,1),-2*sqrt(z_res_uncert(1:to, i+1 + (i-1)*state_size)),'-');
    	plot(z_res_uncert(1:to,1),2.576*sqrt(z_res_uncert(1:to, i+1 + (i-1)*state_size)),'-');
    	plot(z_res_uncert(1:to,1),-2.576*sqrt(z_res_uncert(1:to, i+1 + (i-1)*state_size)),'-');
    	
        xlabel 'step';
    	ylabel 'value';
    	title (['trajectory meas residual with uncert element ',num2str(i)]);
	
    end
    
    
    % est prior and post state
    figures.est_prior_plus_post_trajectories = figure;
            
    %[arr_source1 arr_uncert] = convert_step_data_2_arr(kf_data.x_prior);
    arr_source1 = prior_val;
    %[arr_source2 arr_uncert] = convert_step_data_2_arr(kf_data.x_post);
    arr_source2 = post_val;
  
    
    [r c] = size(arr_source1);
    state_size = c-1;
    real_state_size = state_size; %for later use in uncertainties..
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_prior_plus_post_meas_trajectory(i) = subplot(fig_height,fig_width,i);
        
        hold on;
	    plot(arr_source1(1:to,1),arr_source1(1:to,i+1),'r-');
        plot(arr_source2(1:to,1),arr_source2(1:to,i+1),'b-');
        
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory prior (r) and post (b) element ',num2str(i)]);
	
    end
    
    
    % est prior state plus meas est 
    figures.est_prior_plus_meas_trajectories = figure;
            
    %[arr_source1 arr_uncert] = convert_step_data_2_arr(kf_data.x_prior);
    %[arr_source2 arr_uncert] = convert_step_data_2_arr(kf_data.z_hat);
    arr_source1 = prior_val;
    arr_source2 = z_hat_val;
 
    
    [r c] = size(arr_source1);
    state_size = c-1;
    real_state_size = state_size; %for later use in uncertainties..
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_prior_plus_meas_trajectory(i) = subplot(fig_height,fig_width,i);
        
        hold on;
	    plot(arr_source1(1:to,1),arr_source1(1:to,i+1),'-');
        plot(arr_source2(1:to,1),arr_source2(1:to,i+1),'o');
        
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory prior state and meas element ',num2str(i)]);
	
    end
    
    
    % est post state plus meas est 
    figures.est_post_plus_meas_trajectories = figure;
            
    %[arr_source1 arr_uncert] = convert_step_data_2_arr(kf_data.x_post);
    %[arr_source2 arr_uncert] = convert_step_data_2_arr(kf_data.z_hat);
    arr_source1 = post_val;
    arr_source2 = z_hat_val;
 
    [r c] = size(arr_source1);
    state_size = c-1;
    real_state_size = state_size; %for later use in uncertainties..
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
    
	    % display data of element i
	    figures.est_post_plus_meas_trajectory(i) = subplot(fig_height,fig_width,i);
        
        hold on;
	    plot(arr_source1(1:to,1),arr_source1(1:to,i+1),'-');
        plot(arr_source2(1:to,1),arr_source2(1:to,i+1),'o');
        
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory post state and meas element ',num2str(i)]);
	
    end
    % figure with prior uncertainties
    
    figures.prior_state_uncertainties = figure;
            
    %[arr_source1 arr_uncert] = convert_step_data_2_arr(kf_data.x_prior);
    arr_source = prior_uncert;
    
    
%    [r c] = size(arr_source);
%    state_size = c-1;
    
    fig_width = real_state_size;
    fig_height = real_state_size;
    
    
    for i=1:state_size*state_size
    
    
	    % display data of element i
	    figures.prior_state_uncertanty(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1))
    	xlabel 'step';
    	ylabel '(co)variance';
    	title (['prior uncertainty in state element ',num2str(i)]);
	
    end
   
    
    
    % figure with post uncertainties
    
    figures.post_state_uncertainties = figure;
            
              
    %[arr_source1 arr_uncert] = convert_step_data_2_arr(kf_data.x_post);
    arr_source = post_uncert;
    
    
%    [r c] = size(arr_source);
%    state_size = c-1;
    
    fig_width = real_state_size;
    fig_height = real_state_size;
    
    
    for i=1:state_size*state_size
    
    
	    % display data of element i
	    figures.post_state_uncertanty(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1))
    	xlabel 'step';
    	ylabel '(co)variance';
    	title (['posterior uncertainty in state element ',num2str(i)]);
	
    end
   
    
    
    %%
    
	 
    % display Kalman Gains
    % convert gain matrices to arrays
    [arr_source arr_uncert] = convert_step_data_2_arr(kf_data.K);
    
    
    % assuming state size = measurement size...
    fig_width = real_meas_size;
    fig_height = real_state_size;
   
    figures.kalman_gains = figure;
  
    

    for i=1:state_size*real_meas_size,
    
    
	    % display data of element i
	    figures.kalman_gain(i) = subplot(fig_height,fig_width,i);
        
	    plot(arr_source(1:to,1),arr_source(1:to,i+1),'o');
       % text(arr_source(:,1),arr_source(:,i+1),'\bullet');
    	xlabel 'step';
    	ylabel 'K';
    	title (['Kalman Gain in element ',num2str(i)]);
	
    end
   
    
    