function [figures] = charts_estimate_mixed(kf_data, steps, figs)

	% display charts for estimates

%    figures = figs;
   	if nargin < 2,
        disp 'plots mixed estimated plots';
        disp 'syntax: figures = charts_estimate_mixed(kf_data, nr_of_steps_to_show, [figures])';
        error('not enough parameters');
    end;
    
    % combine prior and posterior data
    [res] = combine_prior_and_post(kf_data);
    [z_hat_val z_hat_uncert] = convert_step_data_2_arr(kf_data.z_hat);
   
    
   
    % mixed prior and posterior state trajectories
    figures.est_prior_state_trajectories = figure;
            
    %[prior_val prior_uncert] = convert_step_data_2_arr(kf_data.x_prior);
    %arr_source = prior_val;
    
   
    [tmp to] = size(res);
    if ~(steps==0), 
        to = steps;
    end;
    
    [r c] = size(res(1).value.from);
    state_size = c-1;
    real_state_size = state_size; %for later use in uncertainties..
    
    fig_width = 1;
    fig_height = state_size;
    
    
    for i=1:state_size
    
         var1 = 1;
         var2 = i+1;
   
        % display data of element i
	    figures.est_prior_state_trajectory(i) = subplot(fig_height,fig_width,i);
        hold on;
        
        for k = 1:to-1, 
            tmp_data(1,1) = res(k).value.from(1,var1);
            tmp_data(1,2) = res(k).value.from(1,var2);
            tmp_data(2,1) = res(k).value.to(1,var1);
            tmp_data(2,2) = res(k).value.to(1,var2);
        
        
            % if there was a measurement, then there was a correction
            % otherwise it was copying the prior
            if (isnan(z_hat_val(k,2))),
                plot(tmp_data(:,1), tmp_data(:,2),'b-');    % copying
            else
                plot(tmp_data(:,1), tmp_data(:,2),'r-');    % correction
            end;
        
            tmp_data(1,1) = tmp_data(2,1);
            tmp_data(1,2) = tmp_data(2,2);
            tmp_data(2,1) = res(k+1).value.from(1,var1);
            tmp_data(2,2) = res(k+1).value.from(1,var2);
        
            plot(tmp_data(:,1), tmp_data(:,2),'b-');
        
        end;
       
	    
	   % plot(arr_source(:,1),arr_source(:,i+1))
    	xlabel 'step';
    	ylabel 'value';
    	title (['estimated trajectory prior state element ',num2str(i)]);
	
    end
    

   
   
    % figure with prior uncertainties
    
    figures.prior_state_uncertainties = figure;
            
    %[arr_source1 arr_uncert] = convert_step_data_2_arr(kf_data.x_prior);
%    arr_source = prior_uncert;
    
    
%    [r c] = size(arr_source);
%    state_size = c-1;
    
    fig_width = real_state_size;
    fig_height = real_state_size;
    
    
    for i=1:state_size*state_size
    
    
	    % display data of element i
	    figures.prior_state_uncertanty(i) = subplot(fig_height,fig_width,i);
        
        var1 = 1;
        var2 = i+1;
   
        % display data of element i
	    hold on;
        
        for k = 1:to-1, 
            tmp_data(1,1) = res(k).uncert.from(1,var1);
            tmp_data(1,2) = res(k).uncert.from(1,var2);
            tmp_data(2,1) = res(k).uncert.to(1,var1);
            tmp_data(2,2) = res(k).uncert.to(1,var2);
        
        
            % if there was a measurement, then there was a correction
            % otherwise it was copying the prior
            if (isnan(z_hat_val(k,2))),
                plot(tmp_data(:,1), tmp_data(:,2),'b-');    % copying
            else
                plot(tmp_data(:,1), tmp_data(:,2),'r-');    % correction
            end;
        
            tmp_data(1,1) = tmp_data(2,1);
            tmp_data(1,2) = tmp_data(2,2);
            tmp_data(2,1) = res(k+1).uncert.from(1,var1);
            tmp_data(2,2) = res(k+1).uncert.from(1,var2);
        
            plot(tmp_data(:,1), tmp_data(:,2),'b-');
        
        end;
       
        
	    %plot(arr_source(:,1),arr_source(:,i+1))
    	xlabel 'step';
    	ylabel '(co)variance';
    	title (['prior and posterior uncertainty in state element ',num2str(i)]);
	
    end
   
    
  
    
    