function [figur] = charts_estimate_multi(kf_data, conv_meas)
% conv_meas indicates if measurements should be converted before display

	% display charts for different variables
   	if nargin < 2,
        disp 'plots multiple estimated variables in a plot';
        disp 'syntax: figures = charts_estimate_multi(kf_data, convert_measurements)';
        error('not enough parameters');
    end;
    
    figur.priors = figure;
    
    % source array:
    [arr post_uncert] = convert_step_data_2_arr(kf_data.x_post);
   
  
    titl = 'estimated x,y trajectory with estimated measurements';
    
    % variable:
    var1 = 2; name1 = 'x(cm)';
    var2 = 3; name2 = 'y(cm)';
    
    % style
    style = '-x';
    
	% nr of subplots:
    fig_width = 1;
    fig_height = 1;
           
	% display estimated path with true measurements
	figures.est_traj = subplot(fig_width,fig_height,1);
	plot(arr(:,var1),arr(:,var2),style);
	xlabel (name1);
	ylabel (name2);
	title (titl);
	
	
    hold on;
	
	% add true measurements
    % nb: measurements are relative to the robots position
    % so, we add the estimate position of the robot, to get
    % the total estimated location of where the robot expects
    % the landmark to be..
    [prior_val prior_uncert] = convert_step_data_2_arr(kf_data.x_prior);
    [z_hat_val z_hat_uncert] = convert_step_data_2_arr(kf_data.z_hat);
    
    if conv_meas == 1,
        test = diff_drive_convert_rel_meas_to_abs(prior_val, z_hat_val);
    else
        test = z_hat_val;
    end;
    
    plot(test(:,2),test(:,3),'bx');
   
  
    
    	
    
    