function [z_real] = diff_drive_get_real_lm_meas_from_map(x_real, R_real)
% [z_real] = diff_drive_get_real_lm_meas_from_map(x_real, R_real)
% returns a vector with the x,y,phi to one landmark that it gets from a
% map. Takes as input the true position of the robot and the true
% measurement noise. 
% assumes that structure 'world' contains types, ids, locations, and
% uncertainties of landmarks that the robot knows.

   global world;
    
   
   [tmp nr_of_lm] = size(world.lm);
   
    
   % find closest lm
   dist_closest = 1000000000;
   
   for lm_id=1:nr_of_lm,
   
        % calculate the x,y,phi to one landmark, without noise
        lm = world.lm(lm_id).pos;
    
        x = (lm.value(1,1) - x_real.value(1,1))*cos( x_real.value(3,1)) + (lm.value(2,1) - x_real.value(2,1))*sin( x_real.value(3,1));
        y = (x_real.value(1,1) - lm.value(1,1))*sin( x_real.value(3,1)) + (lm.value(2,1) - x_real.value(2,1))*cos( x_real.value(3,1));
        phi = lm.value(3,1) - x_real.value(3,1);

        dist = sqrt(x^2+y^2);
    
        if dist<dist_closest,    % NB: takes closest with highest id! 
            x_closest = x;
            y_closest = y;
            phi_closest = phi;
            id_closest = lm_id;
            dist_closest = dist;
        end;
    end;
    
    z_real.value = [x_closest;y_closest;phi_closest];

    % we add the id of the landmark to the measurement
    z_real.type = 1;
    z_real.id = id_closest;
    
    
    % add sensor specific noise
    [m n] = size(R_real.uncert);
    for i=1:m, z_real.value(i,1) = z_real.value(i,1) + sqrt(R_real.uncert(i,i))*randn; end;

                    
    % copy uncertainty. uncertainty is simply measurement uncertainty
    % uncertainty on map does not have to do with uncertainty in
    % measurement
    for i=1:m, for j=1:m, z_real.uncert(i,j) = R_real.uncert(i,j); end; end;
    
    
    % check if visible
    %  global one_lm_meas_para;
    
    
    
    global one_lm_meas_para;
    if (dist_closest > one_lm_meas_para.max_dist),
        
       z_real.value(1,1) = NaN;
       z_real.value(2,1) = NaN;
       z_real.value(3,1) = NaN;
        
       [m n] = size(R_real.uncert);
        
        % copy uncertainty
        for i=1:m, for j=1:m, z_real.uncert(i,j) = NaN; end; end;

    end;
    
   % disp 'real measurement made.';

