function [x_post, C_x_post, K, z_hat, C_z_hat, z_res, C_res] = ekf_meas_update_step(prob, true_z)

   
    
    % extract parameters and Jacobians etc from model
    [h n para J] = get_model_settings(prob, prob.model.meas);
   
    
    % assuming the first argument is the old state (!)
    C_x_prior = para(1).uncert;
    x_prior = para(1).value;
    
    
    % get last made real measurement 
    [z] = true_z.value;
    
 
        
    % make measurement prediction
    [z_hat] = feval(h, para(:).value);
    
    
    % calculate measurement residual
    [z_res] = z - z_hat;
    
    % ===============================================================
    % make sure that the orientation stays as close to 0 as possible
    % assumes z_res(3,1) contains orientation!
    % ===============================================================
    if isfield(prob.model.meas,'convert_orientation'), % if conversion needed
        if z_res(prob.model.meas.convert_orientation,1) > pi,
            z_res(prob.model.meas.convert_orientation,1) = z_res(prob.model.meas.convert_orientation,1) - 2*pi;
        end;
    end;
    
    % calculate uncertainty in residual
    % this is noise in prediction plus measurement
    for i=1:n,
        C_tmp(i).influence = J(i).m*para(i).uncert*(J(i).m');
    end;
        
    C_res = C_tmp(1).influence;
    if n>1, for i=2:n, C_res = C_res + C_tmp(i).influence; end; end;
    

    % the uncertainty in the prediction is the uncertainty in residual..
    C_z_hat = C_res;
    
    % ready for updates...
     
    % calculate Kalman gain   
    if rcond(inv(C_res)) < 6.096580e-18,
        disp 'Inverse warning!' ;
        
   %     pause;
    end;
    
    
    % check if this is first time a measurement is incorporated
    global uniform_belief;
    if (uniform_belief == 1), %if this is the case, then K = H^(-1), to make sure all information is incorporated
        
        K = J(1).m^(-1); % assuming J(1) corresponds with jacobian of h w.r.t x...
        
        % update state
        x_post = x_prior + K*z_res;
        
%        delta_x = true_z.value;
%        for bla = 2:n,
%            delta_x = delta_x - J(bla).m*para(bla).value;
%        end;
%        x_post = x_prior + K*delta_x;
%        x_post = true_z.value;
%        for bla = 2:n,
%            x_post = x_post + para(bla).value;
%        end;
            
        % and uncertainty
        C_x_post = C_x_prior - K*(J(1).m)*C_x_prior;

        % correct uncertainty with measurement uncert
        for i=2:n, % assuming 1 = P^-...        
 %           C_x_post = C_x_post + J(i).m*para(i).uncert*(J(i).m');  <--
 %           incorrect!
 
             C_x_post = C_x_post + K*(J(i).m*para(i).uncert*(J(i).m'))*K';
        end;
    
        
        uniform_belief = 0; % no longer uniform after incorporation
                
    else
        
        K = (C_x_prior*((J(1).m)'))*( inv(C_res) );
        
        % update state
        x_post = x_prior + K*z_res;
    
        % and uncertainty
        C_x_post = C_x_prior - K*(J(1).m)*C_x_prior;

    
    end;
    
    
    

   
    
    % done.
    pr 'ekf measurement update performed.';

