function [x_post, C_x_post, K, z_hat, C_z_hat, z_res, C_res, z, z_hat_new, C_z_hat_new] = iekf_meas_update_step(prob, true_z)

    % extract parameters and Jacobians etc from model
    [h n para J] = get_model_settings(prob, prob.model.meas);
   
    % assuming the first argument is the old state (!)
    C_x_prior = para(1).uncert;
    x_prior = para(1).value;
 
    
    % get last made real measurement 
    [z] = true_z.value;
    
    
     
    % make measurement prediction
    [z_hat_original] = feval(h, para(:).value);
    
    
    % iteration part
    % 
    go_on = 1;
    
    while go_on > 0, % find best state estimate match
        
        % get latest state estimate (amongst others)
        % extract parameters and Jacobians etc from model
        [h n para J] = get_model_settings(prob, prob.model.meas);
  
        
        % make measurement prediction
        [z_hat] = feval(h, para(:).value);
    
    
        % calculate measurement residual
        [z_res] = z - z_hat;
    
        % ===============================================================
        % make sure that the orientation stays as close to 0 as possible
        % assumes z_res(3,1) contains orientation!
        % ===============================================================
        if isfield(prob.model.meas,'convert_orientation'), % if conversion needed
            if z_res(prob.model.meas.convert_orientation,1) > pi,
                z_res(prob.model.meas.convert_orientation,1) = z_res(prob.model.meas.convert_orientation,1) - 2*pi;
         end;
        end;
    
    
    
    
    
        % calculate uncertainty in residual
        % this is noise in prediction plus measurement
        for i=1:n,
            C_tmp(i).influence = J(i).m*para(i).uncert*(J(i).m');
        end;
        
        C_res = C_tmp(1).influence;
        if n>1, for i=2:n, C_res = C_res + C_tmp(i).influence; end; end;
    
        % the uncertainty in the prediction is the uncertainty in residual..
        C_z_hat = C_res;
  
        % ready for updates...
        
        
        % check if this is first time a measurement is incorporated
        global uniform_belief;
        if (uniform_belief == 1), %if this is the case, then K = H^(-1), to make sure all information is incorporated
        
            K = J(1).m^(-1); % assuming J(1) corresponds with jacobian of h w.r.t x...
        
            % update state
            x_post = prob.model.state_est.value + K*z_res;
            
        
            uniform_belief = 1; % no longer uniform after incorporation
                
        else
        
        
        
            % calculate Kalman gain   
            K = (C_x_prior*((J(1).m)'))*( C_res^(-1) );
    
        
            % update state
            x_post = prob.model.state_est.value + K*z_res;
            
            
        end;
        
        
            
        % see if difference is significant with last estimate
        x_diff = prob.model.state_est.value - x_post;
        
        
        
        signif = 0;
        
        [n m] = size(x_diff);
        
        for i=1:n,
            if abs(x_diff(i,1)) > 0.00005,
                signif = 1;
            end;
        end;
        
        % save x_post in prob structure, such that a new 
        % iteration with x_post as new state estimate can be done
        prob.model.state_est.value = x_post;
    
        interm_states.state(go_on,1) = go_on;
        interm_states.state(go_on,2) = x_post(1,1);
        interm_states.state(go_on,3) = x_post(2,1);
        interm_states.state(go_on,4) = x_post(3,1);
        interm_states.diff(go_on,1) = go_on;
        interm_states.diff(go_on,2) = x_diff(1,1);
        interm_states.diff(go_on,3) = x_diff(2,1);
        interm_states.diff(go_on,4) = x_diff(3,1);
        
        % iterate until state does not change significantly
        % or until x rounds made
        if (go_on < 100 && signif == 1),
            go_on = go_on + 1;
        else
            go_on = 0;
        end;
        
        display_info(prob);
    %    disp 'press key..';
       % pause;
        
    end;
    
    result_state = x_post;
    
    % now use found best estimate from where measurement was made
    % to include measurement into original prior state estimate
    % using linearization at best estimate and original measurement
    
    % compute z_hat of prior state estimate, given new linearization
    [z_res] = J(1).m*(result_state - x_prior);
    
    
    if uniform_belief==1,
        
        x_post = x_prior + K*z_res;
          
        C_x_post = C_x_prior - K*(J(1).m)*C_x_prior;

        % correct uncertainty with measurement uncert
        for i=2:n, % assuming 1 = P^-...
        %   C_x_post = C_x_post + J(i).m*para(i).uncert*(J(i).m');
           C_x_post = C_x_post + K*(J(i).m*para(i).uncert*(J(i).m'))*K';
        end;
        
    
        uniform_belief = 0;
    else
    
        x_post = x_prior + K*z_res;
        
        % now also update the uncertainty
        C_x_post = C_x_prior - K*(J(1).m)*C_x_prior;
    end;


    [z_hat] = [z_hat_original];
    
    
    % return fresh measurement estimate for multiple beliefs 
    prob.model.state_est.value = x_post;
    prob.model.state_est.uncert = C_x_post;
   
    [h n para J] = get_model_settings(prob, prob.model.meas);
   
    z_hat_new = feval(h, para(:).value);
    
    % calculate uncertainty in residual
    % this is noise in prediction plus measurement
    for i=1:n,
        C_tmp(i).influence = J(i).m*para(i).uncert*(J(i).m');
    end;
        
    C_z_hat_new = C_tmp(1).influence;
    if n>1, for i=2:n, C_res = C_z_hat_new + C_tmp(i).influence; end; end;
    
     
    
    % done.
    pr 'iekf measurement update performed.';

