
function [problem, x_prior, x_post,  K, z_hat, z_res] = make_ekf_step(problem)

    % given a problem model, estimates the state using 
    % the Linear Kalman Filter equations

    % start ekf with time update
  %  pr 'perform ekf time update';

   
    % execute time update
    %[x_prior(1).value c_x_prior(1).uncert] = ekf_time_update_step(problem);
    [x_prior(1).value x_prior(1).uncert] = ekf_time_update_step(problem);

    % and save the result
    problem.model.state_est.value = x_prior(1).value;
    problem.model.state_est.uncert = x_prior(1).uncert;

    % and result to ui
  %  display_info(problem);


    
    
    % next, perform the measurement update
  %  pr 'perform ekf measurement update';

    
    % check if there is a measurement
    zs = problem.env.meas.result.data;
    
    [tmp nr_of_measurements] = size(zs);
    
    no_meas = 0;
    
    if nr_of_measurements==1, % then either there is real or dummy
        
        if isnan( zs(1).value(1,1) ), % if first element is NaN, then dummy
            no_meas = 1;
        else
            no_meas = 0; % otherwise it is a real measurement
        end;
        
    end;
        

    %if no real measurement, then copy prior info to posterior and return
    if no_meas,
        
        x_post(1).value = x_prior(1).value;
        x_post(1).uncert = x_prior(1).uncert;
        
        % make K with NaNs..
        [r c] = size(zs(1).value);
        [r2 c2] = size(x_post(1).value);
        for a=1:r,
            
            z_hat(1).value(a,1) = NaN;   % create also z_hat with NaN..
            z_res(1).value(a,1) = NaN;   % and z_res..
            
            for c=1:r,
                z_res(1).uncert(r,c) = NaN; % and fill C_res with NaNs..
                z_hat(1).uncert(r,c) = NaN;
            end
            
            for b=1:r2,
                K(1).value(b,a) = NaN;
                K(1).uncert(b,a) = NaN; % filling...
            end
        end
        
        
        
    else  % walk through different measurements
        
       
        for kk=1:nr_of_measurements,
            
            z_tr.value = zs(kk).value;
            z_tr.uncert = zs(kk).uncert;
            
            problem.model.meas.curr_meas = zs(kk);  % save meas, such that parameters can be formed (ie landmark detection)
            
            % save prior state and uncert
            % if there is one measurement, then we already did this,
            % if there are more, we save one prior for every measurement
            if (kk>1), % prior k=1 is already filled in above
                
                x_prior(kk).value = x_post(kk-1).value;
                x_prior(kk).uncert = x_post(kk-1).uncert;
                
            end;
            
            % execute measurement update
            [x_post(kk).value x_post(kk).uncert K(kk).value z_hat(kk).value z_hat(kk).uncert z_res(kk).value z_res(kk).uncert] = ekf_meas_update_step(problem, z_tr);

            % and save the result
            problem.model.state_est.value = x_post(kk).value;
            problem.model.state_est.uncert = x_post(kk).uncert;
            
            K(kk).uncert = K(kk).value; %filling for compatibility..

        end;
        
        problem.model.meas.curr_meas = [];
        
    end
    
    
    
   
    % and result to ui
  %  display_info(problem);
    
    % ekf is done
  %  pr 'ekf step performed.';
