function [result] = run_x_cycles(prob, kf_method, xje, run_method, data, from)

% performs x cycles of process execution and KF
%
% run_method = 1: both process and KF
% run_method = 2: only process
% run_method = 3: only KF
% run_method = 4: make new measurements only

    result = data;
    
    % for every step...
    for i=from:(from-1)+xje
        
  %      pr (strcat('step: ',num2str(i)));
        
        % if run only KF or measure, then initialize pre-generated data in structure
        if (run_method==3 || run_method==4 ) 
            
            % store control
            [n m] = size(data.control);
            for j=2:m,
                prob.env.control.result.value(j-1,1) = data.control(i,j);
            end;
        
            % store process
            [n m] = size(data.process);
            for j=2:m,
                prob.env.process.result.value(j-1,1) = data.process(i,j);
                
                prob.env.state.value(j-1,1) = data.process(i,j);
            end;

            
          
            % store measurements, restores multiple measurements
            prob.env.meas.result = data.meas(i);
            
           
            
        end;
        
        % perform one kf step with the stored data
        [prob ctrl x z x_prior x_post K z_hat z_res ] = run_one_cycle(prob, kf_method, run_method);

        
        % save true data in structure if not run_method is KF only..        
        if (run_method==1 || run_method==2)
            
            % save control
            [n m] = size(ctrl.value);
            result.control(i,1) = i;
            for j=1:n,
                result.control(i,j+1) = ctrl.value(j,1);
            end;
        
            % save true new state
            [n m] = size(x.value);
            result.process(i,1) = i;
            for j=1:n,
                result.process(i,j+1) = x.value(j,1);
            end;
            
                  
            % save true measurement
         %   [n m] = size(z.value);
         %   result.meas(i,1) = i;
         %   for j=1:n,
         %       result.meas(i,j+1) = z.value(j,1);
         %   end;
            
  
        end;
        
        if (run_method == 4 || run_method==1 || run_method==2) % only save measurement            
                  
            % save true measurement
            result.meas(i,1).step = i;
            result.meas(i,1).data = z;    % notice that this is a struct which can contain multiple measurements!
           
        end;
        
        
        % save kf data if not run_method is process only
        if (run_method==1 || run_method==3)

            % save prior and posterior data
            result.x_prior(i,1).step = i;
            result.x_prior(i,1).data = x_prior;
            
            result.x_post(i,1).step = i;
            result.x_post(i,1).data = x_post;

            % save z_hat and z_res
            result.z_hat(i,1).step = i;
            result.z_hat(i,1).data = z_hat;
            
            result.z_res(i,1).step = i;
            result.z_res(i,1).data = z_res;
            
           

            % save kalman gain
            result.K(i,1).step = i;
            result.K(i,1).data = K;
            
        end;
        
        %pr 'press a key..';
        %pause;
    end;
    