function [result] = run_x_cycles(prob, kf_method, xje, run_method, data, from)

% performs x cycles of process execution and KF
%
% run_method = 1: both process and KF
% run_method = 2: only process
% run_method = 3: only KF
% run_method = 4: make new measurements only

    result = data;
    
    % for every step...
    for i=from:(from-1)+xje
        
        pr (strcat('step: ',num2str(i)));
        
        % if run only KF or measure, then initialize pre-generated data in structure
        if (run_method==3 || run_method==4 ) 
            
            % store control
            [n m] = size(data.control);
            for j=2:m,
                prob.env.control.result.value(j-1,1) = data.control(i,j);
            end;
        
            % store process
            [n m] = size(data.process);
            for j=2:m,
                prob.env.process.result.value(j-1,1) = data.process(i,j);
                
                prob.env.state.value(j-1,1) = data.process(i,j);
            end;

          
            % store measurement
            [n m] = size(data.meas);
            for j=2:m,
                prob.env.meas.result.value(j-1,1) = data.meas(i,j);
            end;
            
        end;
        
        % perform one kf step with the stored data
        [prob ctrl x z x_prior c_x_prior x_post c_x_post K z_hat C_z_hat z_res C_res] = run_one_cycle(prob, kf_method, run_method);

        
        % save true data in structure if not run_method is KF only..        
        if (run_method==1 || run_method==2)
            
            % save control
            [n m] = size(ctrl.value);
            result.control(i,1) = i;
            for j=1:n,
                result.control(i,j+1) = ctrl.value(j,1);
            end;
        
            % save true new state
            [n m] = size(x.value);
            result.process(i,1) = i;
            for j=1:n,
                result.process(i,j+1) = x.value(j,1);
            end;
            
                  
            % save true measurement
         %   [n m] = size(z.value);
         %   result.meas(i,1) = i;
         %   for j=1:n,
         %       result.meas(i,j+1) = z.value(j,1);
         %   end;
            
  
        end;
        
        if (run_method == 4 || run_method==1 || run_method==2) % only save measurement            
                  
            % save true measurement and uncert
            [n m] = size(z.value);
            index = 0;
            
            result.meas(i,1) = i;
            result.C_meas(i,1) = i;
            
            for j=1:n,
                
              result.meas(i,j+1) = z.value(j,1);
              
              for k=1:n, % save uncertainty
                  
                  index = index+1;
                  result.C_meas(i, index+1) = z.uncert(j,k);
                  
              end;
              
            end;
            
            
            
        end;
        
        
        % save kf data if not run_method is process only
        if (run_method==1 || run_method==3)

            % save prior and posterior info
            index = 0;
            [n m] = size(x_prior);
        
            % first index is step number
            result.x_prior(i,1) = i;
            result.x_post(i,1) = i;
            result.c_x_prior(i,1) = i;
            result.c_x_post(i,1) = i;
        
            for j=1:n,
                result.x_prior(i,j+1) = x_prior(j,1);
                result.x_post(i,j+1) = x_post(j,1);
            
                for k=1:n,
                    index = index+1;
                
                    result.c_x_prior(i, index+1) = c_x_prior(j,k);
                    result.c_x_post(i, index+1) = c_x_post(j,k);
                end;
            end;
        
            % save z_hat and z_res
            index = 0;
            [n m] = size(z_hat);
            
            result.z_hat(i,1) = i;
            result.z_res(i,1) = i;
            result.C_res(i,1) = i;
            result.C_z_hat(i,1) = i;
            
            for j=1:n,
                result.z_hat(i,j+1) = z_hat(j,1);
                result.z_res(i,j+1) = z_res(j,1);
                
                for k=1:n,
                    index = index+1;
                    result.C_res(i,index+1) = C_res(j,k);
                    result.C_z_hat(i,index+1) = C_z_hat(j,k);
                end;
            end;
            
            
            % save kalman gain
           % index = 1;
            %[n m] = size(K);
            
            result.K(i,1).step = i;
            result.K(i,1).matrix = K;
            
        end;
        
        %pr 'press a key..';
        %pause;
    end;
    